package GII::Event;

=head1 NAME

GII::Event - Perl object encapsulating LibGII gii_event_t structure.

=head1 SYNOPSIS

  use GII::Event;

  $ev = new GII::Event;

=head1 DESCRIPTION

The GII::Event class is a parent for subclasses that implement methods
for specific kinds of events.  It supplies access methods for the 
COMMON_DATA (the .any union member) area of the event.  

=cut

use strict;
use Carp;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $AUTOLOAD %ClassMap);
use GII::Event::Command;
use GII::Event::Expose;
use GII::Event::Key;
use GII::Event::PButton;
use GII::Event::PMove;
use GII::Event::Valuator;

require Exporter;
require DynaLoader;
require AutoLoader;

@ISA = qw(Exporter DynaLoader);
$VERSION = '0.01';

=head1 Exported Constants

evNothing
evCommand
evInformation
evExpose
evKeyPress
evKeyRelease
evKeyRepeat
evPtrRelative
evPtrAbsolute
evPtrButtonPress
evPtrButtonRelease
evValRelative
evValAbsolute
emNothing
emCommand
emInformation
emExpose
emKeyPress
emKeyRelease
emKeyRepeat
emKey
emKeyboard
emPtrRelative
emPtrAbsolute
emPtrButtonPress
emPtrButtonRelease
emPtrMove
emPtrButton
emPointer
emValRelative
emValAbsolute
emValuator
emZero
emAll
EV_ORIGIN_NONE
EV_ORIGIN_SENDEVENT
EV_TARGET_ALL

=cut

@EXPORT = qw(
evNothing
evCommand
evInformation
evExpose
evKeyPress
evKeyRelease
evKeyRepeat
evPtrRelative
evPtrAbsolute
evPtrButtonPress
evPtrButtonRelease
evValRelative
evValAbsolute
emNothing
emCommand
emInformation
emExpose
emKeyPress
emKeyRelease
emKeyRepeat
emKey
emKeyboard
emPtrRelative
emPtrAbsolute
emPtrButtonPress
emPtrButtonRelease
emPtrMove
emPtrButton
emPointer
emValRelative
emValAbsolute
emValuator
emZero
emAll
EV_ORIGIN_NONE
EV_ORIGIN_SENDEVENT
EV_TARGET_ALL
);

sub AUTOLOAD {
    $AutoLoader::AUTOLOAD = $AUTOLOAD;
    goto &AutoLoader::AUTOLOAD;
}

bootstrap GII::Event $VERSION;

=head1 Global Variable

GII::Event supplies a global hash, mapping LibGII event types to object 
classes, named GII::Event::ClassMap.  This is used to choose the 
right subclass in which to place an event received by a GII:: object 
or created by the GII::Event constructor.

Subclasses not distributed with the base GII:: modules are welcome to 
modify this hash to hook their class into this automagical mechanism;
but must store/restore the previously existing value if there was one.

=cut


%ClassMap = (
	     &evNothing			=> 'GII::Event',
	     &evCommand			=> 'GII::Event::Command',
	     &evInformation		=> 'GII::Event::Command',
	     &evExpose			=> 'GII::Event::Expose',
	     &evKeyPress		=> 'GII::Event::Key',
	     &evKeyRelease      	=> 'GII::Event::Key',
	     &evKeyRepeat		=> 'GII::Event::Key',
	     &evPtrRelative		=> 'GII::Event::PMove',
	     &evPtrAbsolute		=> 'GII::Event::PMove',
	     &evPtrButtonPress		=> 'GII::Event::PButton',
	     &evPtrButtonRelease	=> 'GII::Event::PButton',
	     &evValRelative		=> 'GII::Event::Valuator',
	     &evValAbsolute		=> 'GII::Event::Valuator'
	     );

=head1 Methods

GII::Event objects support the following methods.  For more methods,
specific to different types of events, see the documentation of the
subclasses of GII::Event.

=head2 new

The GII::Event constructor creates a generic ggi_event structure
when called with no arguments.  If given arguments, the first is
assumed to be an event type, and the event is subclassed to 
the appropriate object class.  The second, if present, is 
a pointer to a pre-allocated ggi_event which will be used
instead of a newly allocated structure.  If the first argument 
is undef, then the object is assumed to contain valid event data
already and the subclass is chosen from the type therein.  
All of the arguments are passed along to any subclass constructors 
that are invoked.

=cut

sub new {
    my ($class) = shift();
    my ($type, $subclass, $this);
    $subclass = $class;
    if (@_ + 0) {
	$type = shift();
	$subclass = $ClassMap{$type} if defined($type);
    }
    if ($class eq $subclass) {
	$this = {};
	bless $this, $class;
	if (@_ + 0) {
	    $this->{obj} = shift();
	    if (defined $type) {
		$this->type($type);
		$this->size(0);
	    }
	    else {
		$subclass = $ClassMap{$this->type};
		unless($class eq $subclass) {
		    my($obj) = $this->{obj};
		    delete($this->{obj}); # Bypass free() in DESTROY
		    return new $subclass undef(), $obj, @_;
		}
	    }
	}
	else {
	    $this->{obj} = $this->_alloc();
	    $this->type(defined($type) ? $type : &evNothing);
	    $this->size(0);
	}
    }
    else {
	$this = new $subclass $type, @_;
    }
    return $this;
}

sub DESTROY {
    my ($this) = shift();
    $this->_free($this->{obj}) unless exists $this->{reused};
}

=head2 size

The size method, with no arguments, returns the size of the event
in bytes.  With an argument, it sets the size and returns the old size.  
The latter should only be done on a non-subclassed GGI::Event object, 
which was created by the user with no specified event type, as the 
actual memory allocation made for any other event may not be big enough 
to contain the number of bytes specified.  It is assumed if you set 
the size of an event you know exactly what you are doing.

The method will only return valid values regardless of the actual
value of the structure member.  It will bump any attempts to set invalid
values to the minimum or maximum event size, as appropriate.  To
be sure that the size structure member is set to a valid, minumum, 
size you can set it to 0.

=cut

sub size {
    my($this) = shift();
    if (@_ + 0) {
	return $this->set_size($this->{obj}, shift());
    }
    return $this->get_size($this->{obj});
}

=head2 type

The type method, with no arguments, returns the type of the event.  
With an argument, it sets the type and returns the old type.
The class of the object is not changed.

=cut

sub type {
    my($this) = shift();
    if (@_ + 0) {
	return $this->set_type($this->{obj}, shift());
    }
    return $this->get_type($this->{obj});
}

=head2 error

The error method, with no arguments, returns the error code contained
in the the event. With an argument, it sets the error code and returns 
the old error code.

Error code macros are defined in the GGI::Error class and may be imported
by explicitly using the GGI::Error module.  A value of zero (GGI_OK)
means no error has occured.

=cut

sub error {
    my($this) = shift();
    if (@_ + 0) {
	return $this->set_error($this->{obj},shift());
    }
    return $this->get_error($this->{obj});
}

=head2 origin

The origin method, with no arguments, returns the device id of the
device originating the event.  With an argument, it sets the origin
device id and returns the old one.

=cut

sub origin {
    my($this) = shift();
    if (@_ + 0) {
	return $this->set_origin($this->{obj},shift());
    }
    return $this->get_origin($this->{obj});
}

=head2 target

The target method, with no arguments, returns the device id of the
device to which the event is to be sent.  With an argument, it 
sets the target device id and returns the old one.

=cut

sub target {
    my($this) = shift();
    if (@_ + 0) {
	return $this->set_target($this->{obj},shift());
    }
    return $this->get_target($this->{obj});
}

=head2 objptr

Returns a C pointer to the encapsulated gii_event object.  For 
internal/subclass use only.

=cut

sub objptr {
    my ($this) = shift();
    return $this->{obj};
}

1;
__END__


=head1 AUTHOR

Brian S. Julin <bri@calyx.com>

=head1 SEE ALSO

GII::Event::Command(3pm), GII::Event::Expose(3pm), GII::Event::Key(3pm), 
GII::Event::PButton(3pm), GII::Event::PMove(3pm), GII::Event::Valuator(3pm), 
GII::(3pm), perl(1).

=cut
